from parsec.text import *
from parsec.combinator import *
from parsec.atom import *
from parsec.state import BasicState

side = string('$')


@Parsec
def escape(state):
    c = state.next()
    if c == "$":
        return "$"
    elif c == "\\":
        return "\\"
    elif c == "{":
        return "{"
    elif c == "}":
        return '}'
    else:
        # 兼容非公式嵌入，对于无法识别的文本返回原文
        return f'\\{c}'


@Parsec
def escaped(state):
    c = state.next()
    if c == '\\':
        return escape(state)
    elif c == "$":
        raise ParsecError(state, "got stop char $")
    else:
        return c


@Parsec
def token_content(state):
    buffer = ""
    while True:
        c = ahead(choice(one, eof))(state)
        if c in ["{", "}", "^", "_", "$", None]:
            return buffer
        else:
            buffer += escaped(state)


@Parsec
def superscript(state):
    c = state.next()
    if c == "{":
        cnt = token_content(state)
        string("}")(state)
        return f"<sup>{cnt}</sup>"
    else:
        return f"<sup>{c}</sup>"


@Parsec
def subscript(state):
    c = state.next()
    if c == "{":
        cnt = token_content(state)
        string("}")(state)
        return f"<sub>{cnt}</sub>"
    else:
        return f"<sub>{c}</sub>"


@Parsec
def content(state):
    buffer = ""
    while True:
        try:
            tran = state.begin()
            c = state.next()
            if c == "$":
                state.commit(tran)
                return buffer
            elif c == "^":
                state.commit(tran)
                buffer += superscript(state)
            elif c == "_":
                state.commit(tran)
                buffer += subscript(state)
            else:
                state.rollback(tran)
                cnt = token_content(state)
                if cnt:
                    buffer += cnt
                else:
                    raise ParsecError(state, f"unexpect content {state.data[state.index]}")
        except ParsecEof:
            raise ParsecError(state, f"expect right $ but eof")


def processor(plain):
    st = BasicState(plain)
    buffer = ""
    while True:
        try:
            tran = st.begin()
            c = one(st)
            if c == "$":
                index = st.index
                try:
                    cnt = content(st)
                    if cnt:
                        buffer += cnt
                    else:
                        buffer += c
                except ParsecError as err:
                    st.commit(tran)
                    buffer += "$"+st.data[index:st.index]

            else:
                st.commit(tran)
                buffer += c

        except ParsecEof:
            return buffer



import json
import logging
import os
import subprocess
import sys
import uuid
import re

from parsec import BasicState, ParsecError

from .exercises.markdown import parse

id_set = set()
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)


def search_author(author_dict, username):
    for key in author_dict:
        names = author_dict[key]
        if username in names:
            return key
    return username


def user_name(md_file, author_dict):
    ret = subprocess.Popen([
        "git", "log", md_file
    ], stdout=subprocess.PIPE)
    lines = list(map(lambda l: l.decode(), ret.stdout.readlines()))
    author_lines = []
    for line in lines:
        if line.startswith('Author'):
            author_lines.append(line.split(' ')[1])
    author_nick_name = author_lines[-1]
    return search_author(author_dict, author_nick_name)


def load_json(p):
    with open(p, 'r', encoding="utf-8") as f:
        try:
            return json.loads(f.read())
        except UnicodeDecodeError:
            logger.info("json 文件 [{p}] 编码错误，请确保其内容保存为 utf-8 或 base64 后的 ascii 格式。")


def dump_json(p, j, exist_ok=False, override=False):
    if os.path.exists(p):
        if exist_ok:
            if not override:
                return
        else:
            logger.error(f"{p} already exist")
            sys.exit(0)

    with open(p, 'w+', encoding="utf8") as f:
        f.write(json.dumps(j, indent=2, ensure_ascii=False))


def ensure_config(path):
    config_path = os.path.join(path, "config.json")
    if not os.path.exists(config_path):
        node = {"keywords": [],
                "keywords_must": [],
                "keywords_forbid": []}
        dump_json(config_path, node, exist_ok=True, override=False)
        return node
    else:
        return load_json(config_path)


def parse_no_name(d):
    p = r'(\d+)\.(.*)'
    m = re.search(p, d)

    try:
        no = int(m.group(1))
        dir_name = m.group(2)
    except:
        sys.exit(0)

    return no, dir_name


def check_export(base, cfg):
    flag = False
    exports = []
    for export in cfg.get('export', []):
        ecfg_path = os.path.join(base, export)
        if os.path.exists(ecfg_path):
            exports.append(export)
        else:
            flag = True
    if flag:
        cfg["export"] = exports
    return flag


class TreeWalker:
    def __init__(
            self, root,
            tree_name,
            title=None,
            log=None,
            authors=None,
            enable_notebook=None,
            ignore_keywords=False
    ):
        self.ignore_keywords = ignore_keywords
        self.authors = authors if authors else {}
        self.enable_notebook = enable_notebook
        self.name = tree_name
        self.root = root
        self.title = tree_name if title is None else title
        self.tree = {}
        self.logger = logger if log is None else log

    def walk(self):
        root = self.load_root()
        root_node = {
            "node_id": root["node_id"],
            "keywords": root.get("keywords", []),
            "children": [],
            "keywords_must": root.get("keywords_must", []),
            "keywords_forbid": root.get("keywords_forbid", [])
        }
        self.tree[root["tree_name"]] = root_node
        self.load_levels(root_node)
        self.load_chapters(self.root, root_node)
        for index, level in enumerate(root_node["children"]):
            level_title = list(level.keys())[0]
            level_node = list(level.values())[0]
            level_path = os.path.join(self.root, f"{index + 1}.{level_title}")
            self.load_chapters(level_path, level_node)
            for index, chapter in enumerate(level_node["children"]):
                chapter_title = list(chapter.keys())[0]
                chapter_node = list(chapter.values())[0]
                chapter_path = os.path.join(
                    level_path, f"{index + 1}.{chapter_title}")
                self.load_sections(chapter_path, chapter_node)
                for index, section_node in enumerate(chapter_node["children"]):
                    section_title = list(section_node.keys())[0]
                    full_path = os.path.join(
                        chapter_path, f"{index + 1}.{section_title}")
                    if os.path.isdir(full_path):
                        self.check_section_keywords(full_path)
                        self.ensure_exercises(full_path)

        tree_path = os.path.join(self.root, "tree.json")
        dump_json(tree_path, self.tree, exist_ok=True, override=True)
        return self.tree

    def sort_dir_list(self, dirs):
        result = [self.extract_node_env(dir) for dir in dirs]
        result.sort(key=lambda item: item[0])
        return result

    def load_levels(self, root_node):
        levels = []
        for level in os.listdir(self.root):
            if not os.path.isdir(level):
                continue
            level_path = os.path.join(self.root, level)
            num, config = self.load_level_node(level_path)
            levels.append((num, config))

        levels = self.resort_children(self.root, levels)
        root_node["children"] = [item[1] for item in levels]
        return root_node

    def load_level_node(self, level_path):
        config = self.ensure_level_config(level_path)
        num, name = self.extract_node_env(level_path)

        result = {
            name: {
                "node_id": config["node_id"],
                "keywords": config["keywords"],
                "children": [],
                "keywords_must": config.get("keywords_must", []),
                "keywords_forbid": config.get("keywords_forbid", [])
            }
        }

        return num, result

    def load_chapters(self, base, level_node):
        chapters = []
        for name in os.listdir(base):
            full_name = os.path.join(base, name)
            if os.path.isdir(full_name):
                num, chapter = self.load_chapter_node(full_name)
                chapters.append((num, chapter))

        chapters = self.resort_children(base, chapters)
        level_node["children"] = [item[1] for item in chapters]
        return level_node

    def load_sections(self, base, chapter_node):
        sections = []
        for name in os.listdir(base):
            full_name = os.path.join(base, name)
            if os.path.isdir(full_name):
                num, section = self.load_section_node(full_name)
                sections.append((num, section))

        sections = self.resort_children(base, sections)
        chapter_node["children"] = [item[1] for item in sections]
        return chapter_node

    def resort_children(self, base, children):
        children.sort(key=lambda item: item[0])
        for index, [number, element] in enumerate(children):
            title = list(element.keys())[0]
            origin = os.path.join(base, f"{number}.{title}")
            posted = os.path.join(base, f"{index + 1}.{title}")
            if origin != posted:
                self.logger.info(f"rename [{origin}] to [{posted}]")
            os.rename(origin, posted)
        return children

    def ensure_chapters(self):
        for subdir in os.listdir(self.root):
            self.ensure_level_config(subdir)

    def load_root(self):
        config_path = os.path.join(self.root, "config.json")
        if not os.path.exists(config_path):
            config = {
                "tree_name": self.name,
                "keywords": [],
                "node_id": self.gen_node_id(),
                "keywords_must": [],
                "keywords_forbid": []
            }
            dump_json(config_path, config, exist_ok=True, override=True)
        else:
            config = load_json(config_path)
            flag, result = self.ensure_node_id(config)
            if flag:
                dump_json(config_path, result, exist_ok=True, override=True)

        return config

    def ensure_level_config(self, path):
        config_path = os.path.join(path, "config.json")
        if not os.path.exists(config_path):
            config = {
                "node_id": self.gen_node_id()
            }
            dump_json(config_path, config, exist_ok=True, override=True)
        else:
            config = load_json(config_path)
            flag, result = self.ensure_node_id(config)
            if flag:
                dump_json(config_path, config, exist_ok=True, override=True)
        return config

    def ensure_chapter_config(self, path):
        config_path = os.path.join(path, "config.json")
        if not os.path.exists(config_path):
            config = {
                "node_id": self.gen_node_id(),
                "keywords": [],
                "keywords_must": [],
                "keywords_forbid": []
            }
            dump_json(config_path, config, exist_ok=True, override=True)
        else:
            config = load_json(config_path)
            flag, result = self.ensure_node_id(config)
            if flag:
                dump_json(config_path, config, exist_ok=True, override=True)
        return config

    def ensure_section_config(self, path):
        config_path = os.path.join(path, "config.json")
        if not os.path.exists(config_path):
            config = {
                "node_id": self.gen_node_id(),
                "keywords": [],
                "children": [],
                "export": [],
                "keywords_must": [],
                "keywords_forbid": []
            }
            dump_json(config_path, config, exist_ok=True, override=True)
        else:
            config = load_json(config_path)
            flag, result = self.ensure_node_id(config)
            if flag:
                dump_json(config_path, result, exist_ok=True, override=True)
        return config

    def ensure_node_id(self, config):
        flag = False
        if "node_id" not in config or \
                not config["node_id"].startswith(f"{self.name}-") or \
                config["node_id"] in id_set:
            new_id = self.gen_node_id()
            id_set.add(new_id)
            config["node_id"] = new_id
            flag = True

        for child in config.get("children", []):
            child_node = list(child.values())[0]
            f, _ = self.ensure_node_id(child_node)
            flag = flag or f

        return flag, config

    def gen_node_id(self):
        return f"{self.name}-{uuid.uuid4().hex}"

    def extract_node_env(self, path):
        try:
            _, dir = os.path.split(path)
            self.logger.info(path)
            number, title = dir.split(".", 1)
            return int(number), title
        except Exception as error:
            self.logger.error(f"目录 [{path}] 解析失败，结构不合法，可能是缺少序号")
            # sys.exit(1)
            raise error

    def load_chapter_node(self, full_name):
        config = self.ensure_chapter_config(full_name)
        num, name = self.extract_node_env(full_name)
        result = {
            name: {
                "node_id": config["node_id"],
                "keywords": config["keywords"],
                "children": [],
                "keywords_must": config.get("keywords_must", []),
                "keywords_forbid": config.get("keywords_forbid", [])
            }
        }
        return num, result

    def load_section_node(self, full_name):
        config = self.ensure_section_config(full_name)
        num, name = self.extract_node_env(full_name)
        result = {
            name: {
                "node_id": config["node_id"],
                "keywords": config.get("keywords", []),
                "children": config.get("children", []),
                "keywords_must": config.get("keywords_must", []),
                "keywords_forbid": config.get("keywords_forbid", [])
            }
        }
        # if "children" in config:
        #     result["children"] = config["children"]
        return num, result

    def ensure_exercises(self, section_path):
        config = self.ensure_section_config(section_path)
        flag = False
        for e in os.listdir(section_path):
            base, ext = os.path.splitext(e)
            _, source = os.path.split(e)
            if ext != ".md":
                continue
            mfile = base + ".json"
            meta_path = os.path.join(section_path, mfile)
            md_file = os.path.join(section_path, e)
            self.ensure_exercises_meta(meta_path, source, md_file)
            export = config.get("export", [])
            if mfile not in export and self.name != "algorithm":
                export.append(mfile)
                flag = True
                config["export"] = export
            with open(md_file, "r", encoding="utf-8") as efile:
                try:
                    data = efile.read()
                except UnicodeDecodeError:
                    logger.error(f"习题 [{md_file}] 编码错误，请确保其保存为 utf-8 编码")
                    sys.exit(1)
                state = BasicState(data)
                try:
                    doc = parse(state)
                except ParsecError as err:
                    index = state.index
                    context = state.data[index - 15:index + 15]
                    logger.error(f"习题 [{md_file}] 解析失败，在位置 {index} [{context}] 附近有格式: [{err}]")

        if flag:
            dump_json(os.path.join(section_path, "config.json"),
                      config, True, True)

        for e in config.get("export", []):
            full_name = os.path.join(section_path, e)
            exercise = load_json(full_name)
            if "exercise_id" not in exercise or exercise.get("exercise_id") in id_set:
                eid = uuid.uuid4().hex
                exercise["exercise_id"] = eid
                dump_json(full_name, exercise, True, True)
            else:
                id_set.add(exercise["exercise_id"])

    def ensure_exercises_meta(self, meta_path, source, md_file):
        _, mfile = os.path.split(meta_path)
        meta = None
        if os.path.exists(meta_path):
            with open(meta_path) as f:
                content = f.read()
            if content:
                meta = json.loads(content)
                if "exercise_id" not in meta:
                    meta["exercise_id"] = uuid.uuid4().hex
                if "notebook_enable" not in meta:
                    meta["notebook_enable"] = self.default_notebook()
                if "source" not in meta:
                    meta["source"] = source
                if "author" not in meta:
                    meta["author"] = user_name(md_file, self.authors)
                if "type" not in meta:
                    meta["type"] = "code_options"

        if meta is None:
            meta = {
                "type": "code_options",
                "author": user_name(md_file, self.authors),
                "source": source,
                "notebook_enable": self.default_notebook(),
                "exercise_id": uuid.uuid4().hex
            }
        dump_json(meta_path, meta, True, True)

    def default_notebook(self):
        if self.enable_notebook is not None:
            return self.enable_notebook
        if self.name in ["python", "java", "c"]:
            return True
        else:
            return False

    def check_section_keywords(self, full_path):
        if self.ignore_keywords:
            return
        config = self.ensure_section_config(full_path)
        if not config.get("keywords", []):
            self.logger.error(f"节点 [{full_path}] 的关键字为空，请修改配置文件写入关键字")
            sys.exit(1)



import os
from sys import path, version
from types import new_class
from .tree import load_json, dump_json


def simple_list_md_load(p):
    with open(p, 'r', encoding='utf-8') as f:
        lines = f.readlines()
        result = []
        for line in lines:
            item = line.strip('\n')
            result.append(item)
        return result


def simple_list_md_dump(p, lines):
    with open(p, 'w', encoding='utf-8') as f:
        f.write('\n'.join(lines))


class ImgWalker():

    def __init__(self, root) -> None:
        self.root = root

    def walk(self):
        for base, dirs, files in os.walk(self.root):
            for file in files:
                if file[-3:] == '.md':
                    md_file = os.path.join(base, file)
                    md_lines = simple_list_md_load(md_file)
                    md_new = []
                    for line in md_lines:
                        new_line = line.replace(
                            '![](./', f'![](https://gitcode.net/csdn/skill_tree_opencv/-/raw/master/{base}/')
                        md_new.append(new_line)
                    md_new.append('')
                    simple_list_md_dump(md_file, md_new)
                    # import sys
                    # sys.exit(0)



#!/usr/bin/env python

from setuptools import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name="skill-tree-parser",
      version="0.0.8",
      description="CSDN Skill Tree Parser",
      long_description=long_description,
      long_description_content_type='text/markdown',
      author="Liu Xin",
      author_email="liuxin@csdn.net",
      url="https://gitcode.net/csdn/skill_tree_parser",
      license="MIT",
      packages=["skill_tree", "skill_tree.exercises"],
      package_dir={
          "skill_tree": "src/skill_tree",
          "skill_tree.exercises": "src/skill_tree/exercises"
      },
      install_requires=[
          "pyparsec",
          "GitPython"
      ],
      classifiers=[
          "Topic :: Utilities",
          "Programming Language :: Python :: 3.5",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8",
          "Programming Language :: Python :: 3.9",
          "Programming Language :: Python :: 3 :: Only",
          "License :: OSI Approved :: MIT License"
      ])



import os
from sys import version
from .tree import load_json, dump_json


def simple_list_md_load(p):
    with open(p, 'r', encoding='utf-8') as f:
        lines = f.readlines()
        result = []
        for line in lines:
            item = line.strip('\n')
            if item.startswith('* '):
                item = item[2:]
            result.append(item)
        return result


class DocWalker():

    def __init__(self, root) -> None:
        self.root = root

    def walk(self):
        root = self.root
        root_config_path = os.path.join(root, 'config.json')
        root_config = load_json(root_config_path)
        doc_path = os.path.join(root, 'doc.json')
        versions = []
        for version_dir in root_config['versions']:
            version_full_dir = os.path.join(root, version_dir)
            version_config_path = os.path.join(version_full_dir, 'config.json')
            if os.path.exists(version_config_path):
                version_config = load_json(version_config_path)

                for benchmark in version_config['benchmarks']:
                    username = benchmark['user_name']
                    benchmark['askme'] = f'https://ask.csdn.net/new?expertName={username}'

                asserts_path = os.path.join(
                    version_full_dir,
                    version_config['asserts']
                )
                version_config['asserts'] = load_json(asserts_path)

                bug_fixes_path = os.path.join(
                    version_full_dir,
                    version_config['bugfixes']
                )
                version_config['bugfixes'] = simple_list_md_load(
                    bug_fixes_path)

                features_path = os.path.join(
                    version_full_dir,
                    version_config['features']
                )

                parts = version_full_dir.split("/")
                version_config['version'] = parts[len(parts)-1]
                version_config['features'] = simple_list_md_load(features_path)
                versions.append(version_config)

        root_config['versions'] = versions
        dump_json(doc_path, root_config, True, True)



import sys

from parsec import one, eof
from parsec.combinator import *
from parsec.text import *
from .market_math import processor

"""
使用 markdown 编写的问题应该总是一个固定格式，即

# 问题标题

问题描述

## 模板

```java
package app;
public class App {
    public static void main(String[] args){
        $code
    }
}
```

如果第一个二级标题为 template，则生成一个可选的 template 字段，template 章节中的代码段会作为代码模板，用于被生成的 notebook 

## aop

AOP（面向切面）章节定义 notebook 生成时插入的 cell ，如果定义了这个二级标题，其下复合定义的章节会插在必要的位置。

### before

这里的章节会写入到答案代码之前。

### after

这里的章节会写入到答案代码之后

## 答案

这里写正确答案。此处的代码会写入到 notebook 中，如过定义了模板章节，会将其合并后生成 notebook。

```java
```

## 选项

### A 选项标题会被忽略

可选的描述

```
//  代码
```

### B

可选的描述

```
//  代码
```

### C

可选的描述

```
//  代码
```

### D

可选的描述

```
//  代码
```

需要注意的是，选项的标题并不会出现在最终的题目中，仅作为编辑过程中的标注。而第一个选项就是答案。其顺序在最终展现时由服务代码进行混淆。

"""


class Paragraph:
    def __init__(self, source="", language=""):
        self.source = source
        self.language = language

    def isEmpty(self):
        return self.source == "" and self.language == ""

    def to_map(self):
        return {"content": self.source, "language": self.language}


class Option:
    def __init__(self, paras=None):
        self.paras = paras if paras else []

    def to_map(self):
        return [p.to_map() for p in self.paras]


class Exercise:
    def __init__(self, title, answer, description, options):
        self.title = title
        self.answer = answer
        self.description = description
        self.options = options


@Parsec
def spaces(state):
    return skip1(space)(state)


def maybe_spaces(state):
    return skip(space)(state)


def inline(state):
    char = one(state)
    if char == '\n':
        raise ParsecError(state, "unexpected newline")
    else:
        return char


@Parsec
def title(state):
    """
    解析问题标题
    """
    parser = string("#").then(spaces).then(many1(inline)).over(string('\n'))
    data = parser(state)
    return "".join(data)


@Parsec
def chapter_title(state):
    """
    解析章标题
    @param state:
    @return:
    """
    parser = string("##").then(spaces).then(many1(inline)).over(string('\n'))
    data = parser(state)
    return "".join(data)


@Parsec
def section_title(state):
    """
    解析节标题
    @param state:
    @return:
    """
    parser = string("###").then(spaces).then(many1(inline)).over(string('\n'))
    data = parser(state)
    return "".join(data)


@Parsec
def paragraph(state):
    """
    解析文字段落
    """
    buffer = ""
    line = many(inline).over(choice(attempt(string("\n")), eof))
    stop = choices(attempt(string("\n")), attempt(string("### ")), string("## "))
    while True:
        maybe_spaces(state)
        result = "".join(line(state))
        data = result
        if data:
            buffer += '\n' + result
        else:
            break
        tran = state.begin()
        try:
            stop(state)
        except ParsecError:
            continue
        finally:
            state.rollback(tran)
        return Paragraph(processor(buffer).strip(), "markdown")

    return Paragraph(processor(buffer), "markdown")


@Parsec
def code(state):
    side = choice(attempt(string("````")), string("```"))
    left = many(attempt(inline)).over(string("\n"))

    side_token = side(state)
    language = ''.join(left(state))

    right = attempt(string("\n"+side_token).over(choice(attempt(string("\n")), eof)))
    buffer = ""
    while True:
        try:
            right(state)
            return Paragraph(buffer, language)
        except ParsecError:
            buffer += one(state)


@Parsec
def desc(state):
    """解析问题或选项描述
    问题描述由若干段落或代码组成，内部结构遵循 markdown 语法
    """
    buffer = []
    parser = choice(attempt(code), paragraph)
    stop = choices(attempt(string("## ")), attempt(string("### ")), eof)
    while True:
        tran = state.begin()
        try:
            stop(state)
            return buffer
        except ParsecError:
            pass
        finally:
            state.rollback(tran)

        buffer.append(parser(state))
        maybe_spaces(state)


def option(state):
    "解析选项"
    parser = attempt(string("###").then(spaces).then(many1(attempt(inline))).over(string('\n')))
    parser(state)
    maybe_spaces(state)
    return Option(desc(state))


def template(state):
    "解析模板，返回对应的模板代码对象，如果解析失败，返回 None 并恢复 state"
    tran = state.begin()
    try:
        title = chapter_title(state)
        if title == "template":
            state.commit(tran)
            maybe_spaces(state)
            return code(state)
        else:
            raise ParsecError(state, "template not found")
    except ParsecError:
        state.rollback(tran)
        return None


@Parsec
def aop_parser(state):
    """
    解析AOP，返回对应的AOP字典，如果解析失败，返回 None 并恢复 state
    @param state:
    @return:
    """
    result = {}
    stop = attempt(chapter_title)
    tt = state.begin()
    try:
        title = chapter_title(state)
        if title == "aop":
            state.commit(tt)
            maybe_spaces(state)
            while True:
                tran = state.begin()
                try:
                    stop(state)
                    state.rollback(tran)
                    if len(result) == 0:
                        return None
                    else:
                        return result
                except ParsecError:
                    state.rollback(tran)
                    maybe_spaces(state)
                    st = section_title(state)
                    maybe_spaces(state)
                    if st == "before":
                        result["before"] = desc(state)
                    elif st == "after":
                        result["after"] = desc(state)
                    else:
                        raise ParsecError(state, f"invalid section {st} in aop chapter")

        else:
            raise ParsecError(state, "aop not found")
    except ParsecError as err:
        state.rollback(tt)
        return None


@Parsec
def parse(state):
    t = title(state)
    maybe_spaces(state)
    try:
        description = desc(state)
    except ParsecError as err:
        raise err
    tmpl = template(state)
    maybe_spaces(state)
    aop = aop_parser(state)
    ct = chapter_title(state)
    if ct == "答案":
        maybe_spaces(state)
        answer = desc(state)
    else:
        raise ParsecError(state, "chapter [答案] is required")
    ct = chapter_title(state)
    if ct == "选项":
        maybe_spaces(state)
        options = []
        while True:
            try:
                opt = option(state)
                options.append(opt)
                maybe_spaces(state)
            except ParsecError as err:
                result = Exercise(t, answer, description, options)
                if tmpl is None:
                    tmpl = template(state)
                if aop is None:
                    aop = aop_parser(state)
                if tmpl is not None:
                    result.template = tmpl
                if aop is not None:
                    result.aop = aop
                return result
    else:
        raise ParsecError(state, "chapter [选项] not found")
